const rl = @import("raylib");
const std = @import("std");
const ecs = @import("zig-ecs");
const EnSysTypes = @import("EntitySystemTypes.zig");
const time = @cImport(@cInclude("time.h"));

pub fn main() anyerror!void {
    const screenWidth = 1280; // set screen size
    const screenHeight = 720;

    rl.initWindow(screenWidth, screenHeight, "AHHHH"); // make window
    defer rl.closeWindow();

    var Aloc = std.heap.ArenaAllocator.init(std.heap.page_allocator); // specify memory allocator
    defer Aloc.deinit();

    var reg = ecs.Registry.init(Aloc.allocator()); // specify Entity Registry
    try SetupECS(&reg); // run Entity Registry setup

    rl.setTargetFPS(60); // set max fps
    var delta: f32 = 1;
    while (!rl.windowShouldClose()) {
        rl.beginDrawing();
        defer rl.endDrawing();
        rl.clearBackground(rl.Color.black);

        try PhysicsSystem(&reg, delta);
        try DrawSystem(&reg);

        rl.drawFPS(10, 10);
        const text = try std.fmt.allocPrintZ(Aloc.allocator(), "{d}", .{delta});
        rl.drawText(text, 500, 10, 11, rl.Color.white);
        delta = 1.0 / @as(f32, @floatFromInt(rl.getFPS()));
        if(delta > 10000 or delta < -10000){
            delta = 0;
        }
    }
}
pub fn SetupECS(reg: *ecs.Registry) !void {
    var player = reg.create();
    reg.add(player, EnSysTypes.Transform{.pos = rl.Vector2.init(50, 50), .rot = 0});
    reg.add(player, EnSysTypes.Rigidbody{.vel = rl.Vector2.init(0, 0)});
    reg.add(player, EnSysTypes.Gravity{.vel = rl.Vector2.init(0, 0)});
    reg.add(player, EnSysTypes.Sprite{.image = rl.loadTexture("src/Images/9k.png")});
}
pub fn DrawSystem(reg: *ecs.Registry) !void {
    var view = reg.view(.{EnSysTypes.Transform, EnSysTypes.Sprite}, .{});
    var iter = view.entityIterator();
    while(iter.next())|entity|{
        const Transform = view.getConst(EnSysTypes.Transform, entity);
        const Sprite = view.getConst(EnSysTypes.Sprite, entity);
        rl.drawTextureEx(Sprite.image, Transform.pos, Transform.rot, 1, rl.Color.white);
    }
}
pub fn PhysicsSystem(reg: *ecs.Registry, delta: f32) !void {
    var view = reg.view(.{EnSysTypes.Transform, EnSysTypes.Rigidbody, EnSysTypes.Gravity}, .{});
    var iter = view.entityIterator();
    while(iter.next())|entity|{
        const Rigidbody = view.get(EnSysTypes.Rigidbody, entity);
        const Transform = view.get(EnSysTypes.Transform, entity);
        const Gravity = view.getConst(EnSysTypes.Gravity, entity);

        Rigidbody.*.vel.x += Gravity.vel.x;
        Rigidbody.*.vel.y += Gravity.vel.y;
        Transform.pos.x += Rigidbody.vel.x * delta;
        Transform.pos.y += Rigidbody.vel.y * delta;
    }
}

